#cython: language_level=3


from ..column cimport register_column_class

from ...view.view cimport View

from obitools3.utils cimport tobytes, \
                             obi_errno_to_exception

from ...capi.obiview cimport obi_get_float_with_elt_name_and_col_p_in_view, \
                             obi_get_float_with_elt_idx_and_col_p_in_view, \
                             obi_set_float_with_elt_name_and_col_p_in_view, \
                             obi_set_float_with_elt_idx_and_col_p_in_view

from ...capi.obitypes     cimport OBI_FLOAT, OBIFloat_NA, obifloat_t



cdef class Column_float(Column):

    @staticmethod
    def new(View    view,
            object  column_name,
            index_t nb_elements_per_line=1,
            object  elements_names=None,
            object  comments=b""):
        
        return Column.new_column(view, column_name, OBI_FLOAT,
                                 nb_elements_per_line=nb_elements_per_line, 
                                 elements_names=elements_names, 
                                 comments=comments)


    cpdef object get_line(self, index_t line_nb):
        cdef obifloat_t value
        cdef object     result 
        value = obi_get_float_with_elt_idx_and_col_p_in_view(self._view.pointer(), self.pointer(), line_nb, 0)
        obi_errno_to_exception(line_nb=line_nb, elt_id=None, error_message="Problem getting a value from a column")
        if value == OBIFloat_NA :
            result = None
        else :
            result = <double> value
        return result
    
    
    cpdef set_line(self, index_t line_nb, object value):
        if value is None :
            value = OBIFloat_NA
        if obi_set_float_with_elt_idx_and_col_p_in_view(self._view.pointer(), self.pointer(), line_nb, 0, <obifloat_t> value) < 0:
            obi_errno_to_exception(line_nb=line_nb, elt_id=None, error_message="Problem setting a value in a column")
 
 
cdef class Column_multi_elts_float(Column_multi_elts):
 
    cpdef object get_item(self, index_t line_nb, object elt_id) :
        cdef obifloat_t value
        cdef object     result
        cdef bytes      elt_name
        if type(elt_id) == int :
            value = obi_get_float_with_elt_idx_and_col_p_in_view(self._view.pointer(), self.pointer(), line_nb, elt_id)
        else :
            elt_name = tobytes(elt_id)
            value = obi_get_float_with_elt_name_and_col_p_in_view(self._view.pointer(), self.pointer(), line_nb, elt_name)
        obi_errno_to_exception(line_nb=line_nb, elt_id=elt_id, error_message="Problem getting a value from a column")
        if value == OBIFloat_NA :
            result = None
        else :
            result = <double> value
        return result
    
    cpdef object get_line(self, index_t line_nb) :
        cdef obifloat_t value
        cdef object     value_in_result
        cdef dict result
        cdef index_t i
        cdef bint all_NA
        result = {}
        all_NA = True
        for i in range(self.nb_elements_per_line) :
            value = obi_get_float_with_elt_idx_and_col_p_in_view(self._view.pointer(), self.pointer(), line_nb, i)
            obi_errno_to_exception(line_nb=line_nb, elt_id=i, error_message="Problem getting a value from a column")
            if value == OBIFloat_NA :
                value_in_result = None
            else :
                value_in_result = <double> value   
            result[self.elements_names[i]] = value_in_result
            if all_NA and (value_in_result is not None) :
                all_NA = False
        if all_NA :
            result = None
        return result
    
    cpdef set_item(self, index_t line_nb, object elt_id, object value) :
        cdef bytes elt_name
        if value is None :
            value = OBIFloat_NA
        if type(elt_id) == int :
            if obi_set_float_with_elt_idx_and_col_p_in_view(self._view.pointer(), self.pointer(), line_nb, elt_id, <obifloat_t> value) < 0 :
                obi_errno_to_exception(line_nb=line_nb, elt_id=elt_id, error_message="Problem setting a value in a column")
        else :
            elt_name = tobytes(elt_id)
            if obi_set_float_with_elt_name_and_col_p_in_view(self._view.pointer(), self.pointer(), line_nb, elt_name, <obifloat_t> value) < 0 :
                obi_errno_to_exception(line_nb=line_nb, elt_id=elt_id, error_message="Problem setting a value in a column")


def register_class():
    register_column_class(OBI_FLOAT, False, Column_float, float)    # TODO why not double?
    register_column_class(OBI_FLOAT, True, Column_multi_elts_float, float)    # TODO why not double?

