#cython: language_level=3


from ..column cimport register_column_class

from ...view.view cimport View

from obitools3.utils cimport tobytes, \
                             obi_errno_to_exception

from ...capi.obiview cimport obi_get_seq_with_elt_name_and_col_p_in_view, \
                             obi_get_seq_with_elt_idx_and_col_p_in_view, \
                             obi_set_seq_with_elt_name_and_col_p_in_view, \
                             obi_set_seq_with_elt_idx_and_col_p_in_view

from ...capi.obitypes     cimport OBI_SEQ, OBISeq_NA

from libc.stdlib cimport free


cdef class Column_seq(Column):

    @staticmethod
    def new(View    view,
            object  column_name,
            index_t nb_elements_per_line=1,
            object  elements_names=None,
            object  comments=b""):
        
        return Column.new_column(view, column_name, OBI_SEQ,
                                 nb_elements_per_line=nb_elements_per_line, 
                                 elements_names=elements_names, 
                                 comments=comments)


    cpdef object get_line(self, index_t line_nb):
        cdef char*  value
        cdef object result
        value = obi_get_seq_with_elt_idx_and_col_p_in_view(self._view.pointer(), self.pointer(), line_nb, 0)
        obi_errno_to_exception(line_nb=line_nb, elt_id=None, error_message="Problem getting a value from a column")
        if value == OBISeq_NA :
            result = None
        else :   # TODO
            try:
                result = <bytes> value
            finally:
                free(value)
        return result


    cpdef set_line(self, index_t line_nb, object value):
        cdef char* value_b
        cdef bytes value_bytes
        
        if value is None :
            value_b = <char*>OBISeq_NA
        else :
            value_bytes = tobytes(value)
            value_b = <char*>value_bytes
        
        if obi_set_seq_with_elt_idx_and_col_p_in_view(self._view.pointer(), self.pointer(), line_nb, 0, value_b) < 0:
            obi_errno_to_exception(line_nb=line_nb, elt_id=None, error_message="Problem setting a value in a column")
 
 
cdef class Column_multi_elts_seq(Column_multi_elts):
 
    cpdef object get_item(self, index_t line_nb, object elt_id) :
        cdef char*  value
        cdef object result
        if type(elt_id) == int :
            value = obi_get_seq_with_elt_idx_and_col_p_in_view(self._view.pointer(), self.pointer(), line_nb, elt_id)
        else :
            elt_name = tobytes(elt_id)
            value = obi_get_seq_with_elt_name_and_col_p_in_view(self._view.pointer(), self.pointer(), line_nb, elt_name)
        obi_errno_to_exception(line_nb=line_nb, elt_id=elt_id, error_message="Problem getting a value from a column")
        if value == OBISeq_NA :
            result = None
        else :
            try:
                result = <bytes> value
            finally:
                free(value)
        return result


    cpdef object get_line(self, index_t line_nb) :
        cdef char*   value
        cdef object  value_in_result
        cdef dict    result
        cdef index_t i
        cdef bint    all_NA
        result = {}
        all_NA = True
        for i in range(self.nb_elements_per_line) :
            value = obi_get_seq_with_elt_idx_and_col_p_in_view(self._view.pointer(), self.pointer(), line_nb, i)
            obi_errno_to_exception(line_nb=line_nb, elt_id=i, error_message="Problem getting a value from a column")
            if value == OBISeq_NA :
                value_in_result = None
            else :
                try:
                    value_in_result = <bytes> value
                finally:
                    free(value)    
            result[self.elements_names[i]] = value_in_result
            if all_NA and (value_in_result is not None) :
                all_NA = False
        if all_NA :
            result = None
        return result


    cpdef set_item(self, index_t line_nb, object elt_id, object value):
        
        cdef bytes elt_name
        cdef char* value_b
        cdef bytes value_bytes
        
        if value is None :
            value_b = <char*>OBISeq_NA
        else :
            value_bytes = tobytes(value)
            value_b = <char*>value_bytes
        
        if type(elt_id) == int :
            if obi_set_seq_with_elt_idx_and_col_p_in_view(self._view.pointer(), self.pointer(), line_nb, elt_id, value_b) < 0:
                obi_errno_to_exception(line_nb=line_nb, elt_id=elt_id, error_message="Problem setting a value in a column")
        
        else :
            elt_name = tobytes(elt_id)
            if obi_set_seq_with_elt_name_and_col_p_in_view(self._view.pointer(), self.pointer(), line_nb, elt_name, value_b) < 0:
                obi_errno_to_exception(line_nb=line_nb, elt_id=elt_id, error_message="Problem setting a value in a column")


def register_class():
    register_column_class(OBI_SEQ, False, Column_seq, bytes)    # TODO str?
    register_column_class(OBI_SEQ, True, Column_multi_elts_seq, bytes)    # TODO str?

